@@ -5,6 +5,8 @@ metacor <- function(cor, n, studlab,
                     comb.fixed=TRUE, comb.random=TRUE,
                     hakn=FALSE,
                     method.tau="DL", tau.preset=NULL, TE.tau=NULL,
+                    tau.common=FALSE,
+                    prediction=FALSE, level.predict=level,
                     method.bias="linreg",
                     title="", complab="", outclab="",
                     byvar, bylab, print.byvar=TRUE
@@ -17,7 +19,7 @@ metacor <- function(cor, n, studlab,
   ##
   mf <- match.call()
   mf$data <- mf$subset <- mf$sm <- NULL
-  mf$level <- mf$level.comb <- NULL
+  mf$level <- mf$level.comb <- mf$level.predict <- mf$prediction <- NULL
   mf$hakn <- mf$method.tau <- mf$tau.preset <- mf$TE.tau <- mf$method.bias <- NULL
   mf[[1]] <- as.name("data.frame")
   mf <- eval(mf, data)
@@ -28,7 +30,7 @@ metacor <- function(cor, n, studlab,
   mf2$cor <- mf2$n <- NULL
   mf2$studlab <- NULL
   mf2$data <- mf2$sm <- NULL
-  mf2$level <- mf2$level.comb <- NULL
+  mf2$level <- mf2$level.comb <- mf2$level.predict <- mf2$prediction <- NULL
   mf2$hakn <- mf2$method.tau <- mf2$tau.preset <- mf2$TE.tau <- mf2$method.bias <- NULL
   mf2$byvar <- NULL
   mf2[[1]] <- as.name("data.frame")
@@ -45,7 +47,8 @@ metacor <- function(cor, n, studlab,
   cor <- mf$cor
   n     <- mf$n
   ##
-  if (!missing(byvar)){
+  missing.byvar <- missing(byvar)
+  if (!missing.byvar){
     byvar.name <- deparse(substitute(byvar))
     byvar <- mf$byvar
   }
@@ -78,7 +81,26 @@ metacor <- function(cor, n, studlab,
   ##
   sm <- c("ZCOR", "COR")[imeth]
   
-
+  
+  ##
+  ## Check for levels of confidence interval
+  ##
+  if (!is.numeric(level) | length(level)!=1)
+    stop("parameter 'level' must be a numeric of length 1")
+  if (level <= 0 | level >= 1)
+    stop("parameter 'level': no valid level for confidence interval")
+  ##
+  if (!is.numeric(level.comb) | length(level.comb)!=1)
+    stop("parameter 'level.comb' must be a numeric of length 1")
+  if (level.comb <= 0 | level.comb >= 1)
+    stop("parameter 'level.comb': no valid level for confidence interval")
+  ##
+  if (!is.numeric(level.predict) | length(level.predict)!=1)
+    stop("parameter 'level.predict' must be a numeric of length 1")
+  if (level.predict <= 0 | level.predict >= 1)
+    stop("parameter 'level.predict': no valid level for confidence interval")
+  
+  
   if (sm=="ZCOR"){
     TE   <- 0.5 * log((1 + cor)/(1 - cor))
     seTE <- sqrt(1/(n - 3))
@@ -89,27 +111,78 @@ metacor <- function(cor, n, studlab,
   }
   
   
+  ##
+  ## Subgroup analysis with equal tau^2:
+  ##
+  if (!missing.byvar & tau.common){
+    if (!is.null(tau.preset))
+      warning("Value for argument 'tau.preset' not considered as argument 'tau.common=TRUE'")
+    ##
+    sm1 <- summary(metagen(TE, seTE, byvar=byvar,
+                           method.tau=method.tau))
+    sQ.w <- sum(sm1$Q.w)
+    sk.w <- sum(sm1$k.w-1)
+    sC.w <- sum(sm1$C.w)
+    ##
+    if (round(sQ.w, digits=18)<=sk.w) tau2 <- 0
+    else tau2 <- (sQ.w-sk.w)/sC.w
+    tau.preset <- sqrt(tau2)
+  }
+  
+  
   if (!is.null(tau.preset))
     m <- metagen(TE, seTE,
                  hakn=hakn, method.tau=method.tau,
-                 tau.preset=tau.preset, TE.tau=TE.tau)
+                 tau.preset=tau.preset, TE.tau=TE.tau,
+                 level=level,
+                 level.comb=level.comb,
+                 prediction=prediction,
+                 level.predict=level.predict)
   else
     m <- metagen(TE, seTE,
                  hakn=hakn, method.tau=method.tau,
-                 TE.tau=TE.tau)
+                 TE.tau=TE.tau,
+                 level=level,
+                 level.comb=level.comb,
+                 prediction=prediction,
+                 level.predict=level.predict)
+  
+  
+  if (m$k>=3){
+    seTE.predict <- sqrt(m$seTE.random^2 + m$tau^2)
+    ci.p <- ci(m$TE.random, seTE.predict, level.predict, m$k-2)
+    p.lower <- ci.p$lower
+    p.upper <- ci.p$upper
+  }
+  else{
+    seTE.predict <- NA
+    p.lower <- NA
+    p.upper <- NA
+  }
+  
+  
+  if (!missing.byvar & tau.common)
+    tau.preset <- NULL
   
   
   res <- list(cor=cor, n=n,
               studlab=studlab,
               TE=TE, seTE=seTE,
               w.fixed=m$w.fixed, w.random=m$w.random,
+              ##
               TE.fixed=m$TE.fixed, seTE.fixed=m$seTE.fixed,
               lower.fixed=m$lower.fixed, upper.fixed=m$upper.fixed,
               zval.fixed=m$zval.fixed, pval.fixed=m$pval.fixed,
               TE.random=m$TE.random, seTE.random=m$seTE.random,
               lower.random=m$lower.random, upper.random=m$upper.random,
               zval.random=m$zval.random, pval.random=m$pval.random,
+              ##
+              seTE.predict=seTE.predict,
+              lower.predict=p.lower, upper.predict=p.upper,
+              level.predict=level.predict,
+              ##
               k=m$k, Q=m$Q, tau=m$tau, se.tau2=m$se.tau2,
+              C=m$C,
               sm=sm,
               method=m$method,
               level=level, level.comb=level.comb,
@@ -120,6 +193,8 @@ metacor <- function(cor, n, studlab,
               method.tau=method.tau,
               tau.preset=tau.preset,
               TE.tau=if (!missing(TE.tau) & method.tau=="DL") TE.tau else NULL,
+              tau.common=tau.common,
+              prediction=prediction,
               method.bias=method.bias,
               title="", complab="", outclab="",
               call=match.call())
